/*
 * Copyright © 2011-2012 Philipp Eichhorn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package lombok.eclipse.handlers;

import static lombok.core.util.ErrorMessages.*;

import lombok.*;
import lombok.core.AnnotationValues;
import lombok.core.AST.Kind;
import lombok.eclipse.EclipseAnnotationHandler;
import lombok.eclipse.EclipseNode;
import lombok.eclipse.handlers.ast.EclipseMethod;

import org.eclipse.jdt.internal.compiler.ast.Annotation;
import org.mangosdk.spi.ProviderFor;

/**
 * Handles the {@link VisibleForTesting} annotation for eclipse.
 */
@ProviderFor(EclipseAnnotationHandler.class)
public class HandleVisibleForTesting extends EclipseAnnotationHandler<VisibleForTesting> {

	@Override
	public void handle(final AnnotationValues<VisibleForTesting> annotation, final Annotation source, final EclipseNode annotationNode) {
		EclipseNode mayBeMethod = annotationNode.up();
		if (mayBeMethod.getKind() == Kind.METHOD) {
			EclipseMethod method = EclipseMethod.methodOf(annotationNode, source);
			if (method.isAbstract()) {
				annotationNode.addError(canBeUsedOnConcreteMethodOnly(VisibleForTesting.class));
				return;
			}
		} else if (mayBeMethod.getKind() != Kind.TYPE) {
			annotationNode.addError(canBeUsedOnClassAndMethodOnly(VisibleForTesting.class));
		}
	}
}
